@@ -21,15 +21,16 @@ use kiss3d::window::{self, Window};
 use serde::Deserialize;
 use std::fmt;
 use std::path::PathBuf;
-use std::sync::{atomic::Ordering::Relaxed, Arc};
+use std::sync::Arc;
 use structopt::StructOpt;
 use tracing::*;
 
 #[cfg(not(target_arch = "wasm32"))]
-use std::sync::atomic::AtomicBool;
+use std::sync::atomic::{AtomicBool, Ordering::Relaxed};
 
 use crate::{
     handle::{JointNamesAndPositions, RobotOrigin, RobotStateHandle},
+    point_cloud::PointCloudRenderer,
     utils::RobotModel,
     Error, Viewer,
 };
@@ -121,12 +122,12 @@ pub struct UrdfViewerApp {
     arms: Vec<k::SerialChain<f32>>,
     names: Vec<String>,
     input_end_link_names: Vec<String>,
-    num_joints: usize,
     index_of_arm: LoopIndex,
     index_of_move_joint: LoopIndex,
     handle: Arc<RobotStateHandle>,
     is_collision: bool,
     ik_constraints: k::Constraints,
+    point_size: f32,
 }
 
 impl UrdfViewerApp {
@@ -168,7 +169,7 @@ impl UrdfViewerApp {
         }
         let arms = end_link_names
             .iter()
-            .filter_map(|name| robot.find(name).map(|j| k::SerialChain::from_end(j)))
+            .filter_map(|name| robot.find(name).map(k::SerialChain::from_end))
             .collect::<Vec<_>>();
         println!("end_link_names = {:?}", end_link_names);
         let names = robot
@@ -190,13 +191,13 @@ impl UrdfViewerApp {
             input_end_link_names,
             urdf_robot,
             robot,
-            num_joints,
             names,
             index_of_arm: LoopIndex::new(num_arms),
             index_of_move_joint: LoopIndex::new(num_joints),
             handle: Arc::new(handle),
             is_collision,
             ik_constraints: k::Constraints::default(),
+            point_size: 10.0,
         })
     }
     pub fn handle(&self) -> Arc<RobotStateHandle> {
@@ -205,18 +206,20 @@ impl UrdfViewerApp {
     pub fn set_ik_constraints(&mut self, ik_constraints: k::Constraints) {
         self.ik_constraints = ik_constraints;
     }
+    pub fn set_point_size(&mut self, point_size: f32) {
+        self.point_size = point_size;
+    }
     fn has_arms(&self) -> bool {
         !self.arms.is_empty()
     }
     fn has_joints(&self) -> bool {
-        self.num_joints > 0
+        !self.names.is_empty()
     }
     pub fn init(&mut self) {
         self.update_robot();
         if self.has_arms() {
-            let mut window = self.window.as_mut().unwrap();
-            self.viewer
-                .add_axis_cylinders(&mut window, "ik_target", 0.2);
+            let window = self.window.as_mut().unwrap();
+            self.viewer.add_axis_cylinders(window, "ik_target", 0.2);
             self.update_ik_target_marker();
         }
     }
@@ -244,14 +247,17 @@ impl UrdfViewerApp {
         self.viewer.update(&self.robot);
         self.update_ik_target_marker();
     }
-    fn request_reload(&mut self) {
-        self.urdf_robot
-            .request_reload(self.input_path.to_str().unwrap());
-    }
-    fn reload_urdf(&mut self, window: &mut Window) {
+    fn reload(&mut self, window: &mut Window, reload_fn: impl FnOnce(&mut RobotModel)) {
+        // remove previous robot
+        self.viewer.remove_robot(window, self.urdf_robot.get());
+
+        // update urdf_robot
+        reload_fn(&mut self.urdf_robot);
+
+        // update robot based on new urdf_robot
         let urdf_robot = self.urdf_robot.get();
-        self.viewer.remove_robot(window, urdf_robot);
         self.robot = urdf_robot.into();
+        self.input_path = PathBuf::from(&self.urdf_robot.path);
         let end_link_names = if self.input_end_link_names.is_empty() {
             self.robot
                 .iter()
@@ -263,21 +269,24 @@ impl UrdfViewerApp {
         };
         self.arms = end_link_names
             .iter()
-            .filter_map(|name| self.robot.find(name).map(|j| k::SerialChain::from_end(j)))
+            .filter_map(|name| self.robot.find(name).map(k::SerialChain::from_end))
             .collect::<Vec<_>>();
-        self.names = self.robot.iter_joints().map(|j| j.name.clone()).collect();
-    }
-    fn reload_and_update(&mut self, window: &mut Window) {
-        if self.urdf_robot.needs_reload.swap(false, Relaxed) {
-            self.reload_urdf(window);
-            self.viewer.add_robot_with_base_dir_and_collision_flag(
-                window,
-                self.urdf_robot.get(),
-                self.input_path.parent(),
-                self.is_collision,
-            );
-            self.update_robot();
+        let names: Vec<_> = self.robot.iter_joints().map(|j| j.name.clone()).collect();
+        if self.names != names {
+            let dof = names.len();
+            self.names = names.clone();
+            let mut current_joint_positions = self.handle.current_joint_positions.write().unwrap();
+            current_joint_positions.names = names;
+            current_joint_positions.positions = vec![0.0; dof];
         }
+
+        self.viewer.add_robot_with_base_dir_and_collision_flag(
+            window,
+            self.urdf_robot.get(),
+            self.input_path.parent(),
+            self.is_collision,
+        );
+        self.update_robot();
     }
 
     /// Handle set_joint_positions request from server
@@ -380,10 +389,10 @@ impl UrdfViewerApp {
                 );
                 self.update_robot();
             }
+            #[cfg(not(target_arch = "wasm32"))]
             Key::L => {
                 // reload
-                self.request_reload();
-                self.reload_and_update(window);
+                self.reload(window, |urdf_robot| urdf_robot.reload());
             }
             Key::R => {
                 if self.has_joints() {
@@ -424,6 +433,7 @@ impl UrdfViewerApp {
 
         let window = self.window.take().unwrap();
         let state = AppState {
+            point_cloud_renderer: PointCloudRenderer::new(self.point_size),
             app: self,
             solver: k::JacobianIkSolver::default(),
             is_ctrl: false,
@@ -446,7 +456,6 @@ impl fmt::Debug for UrdfViewerApp {
             .field("arms", &self.arms)
             .field("names", &self.names)
             .field("input_end_link_names", &self.input_end_link_names)
-            .field("num_joints", &self.num_joints)
             .field("index_of_arm", &self.index_of_arm)
             .field("index_of_move_joint", &self.index_of_move_joint)
             .field("handle", &self.handle)
@@ -466,40 +475,60 @@ struct AppState {
     is_shift: bool,
     last_cur_pos_y: f64,
     last_cur_pos_x: f64,
+    point_cloud_renderer: PointCloudRenderer,
 }
 
 impl AppState {
-    fn handle_request(&mut self) {
+    fn handle_request(&mut self, window: &mut Window) {
         let handle = self.app.handle();
 
-        // Joint positions for server
-        if let Some(ja) = handle.take_target_joint_positions() {
-            match self.app.set_joint_positions_from_request(&ja) {
-                Ok(_) => {
-                    self.app.update_robot();
-                }
-                Err(err) => {
-                    error!("{}", err);
-                }
+        if let Some(robot) = handle.take_robot() {
+            self.app.reload(window, |urdf_robot| *urdf_robot = robot);
+        }
+
+        if let Some(points) = handle.take_point_cloud() {
+            if points.points.len() == points.colors.len() {
+                self.point_cloud_renderer
+                    .insert(points.id, &points.points, &points.colors);
+            } else {
+                warn!(
+                    "points={},colors={}",
+                    points.points.len(),
+                    points.colors.len()
+                );
             }
         }
-        handle.current_joint_positions.write().unwrap().positions =
-            self.app.robot.joint_positions();
 
-        // Robot orientation for server
-        if let Some(ro) = handle.take_target_robot_origin() {
-            self.app.set_robot_origin_from_request(&ro);
-            self.app.update_robot();
+        if self.app.has_joints() {
+            // Joint positions for server
+            if let Some(ja) = handle.take_target_joint_positions() {
+                match self.app.set_joint_positions_from_request(&ja) {
+                    Ok(_) => {
+                        self.app.update_robot();
+                    }
+                    Err(err) => {
+                        error!("{}", err);
+                    }
+                }
+            }
+            handle.current_joint_positions.write().unwrap().positions =
+                self.app.robot.joint_positions();
+
+            // Robot orientation for server
+            if let Some(ro) = handle.take_target_robot_origin() {
+                self.app.set_robot_origin_from_request(&ro);
+                self.app.update_robot();
+            }
+            let mut cur_ro = handle.current_robot_origin.write().unwrap();
+            let o = self.app.robot.origin();
+            for i in 0..3 {
+                cur_ro.position[i] = o.translation.vector[i];
+            }
+            cur_ro.quaternion[0] = o.rotation.quaternion().w;
+            cur_ro.quaternion[1] = o.rotation.quaternion().i;
+            cur_ro.quaternion[2] = o.rotation.quaternion().j;
+            cur_ro.quaternion[3] = o.rotation.quaternion().k;
         }
-        let mut cur_ro = handle.current_robot_origin.write().unwrap();
-        let o = self.app.robot.origin();
-        for i in 0..3 {
-            cur_ro.position[i] = o.translation.vector[i];
-        }
-        cur_ro.quaternion[0] = o.rotation.quaternion().w;
-        cur_ro.quaternion[1] = o.rotation.quaternion().i;
-        cur_ro.quaternion[2] = o.rotation.quaternion().j;
-        cur_ro.quaternion[3] = o.rotation.quaternion().k;
     }
 }
 
@@ -514,8 +543,6 @@ impl window::State for AppState {
             return;
         }
 
-        self.app.reload_and_update(window);
-
         self.app.viewer.draw_text(
             window,
             HOW_TO_USE_STR,
@@ -523,6 +550,7 @@ impl window::State for AppState {
             &na::Point2::new(2000.0, 10.0),
             &na::Point3::new(1f32, 1.0, 1.0),
         );
+        self.handle_request(window);
         if self.app.has_joints() {
             self.app.viewer.draw_text(
                 window,
@@ -534,8 +562,6 @@ impl window::State for AppState {
                 &na::Point2::new(10f32, 20.0),
                 &na::Point3::new(0.5f32, 0.5, 1.0),
             );
-
-            self.handle_request();
         }
         if self.app.has_arms() {
             let name = &self
@@ -573,8 +599,8 @@ impl window::State for AppState {
                 &na::Point3::new(0.9f32, 0.5, 1.0),
             );
         }
+
         for mut event in window.events().iter() {
-            self.app.reload_and_update(window);
             match event.value {
                 WindowEvent::MouseButton(_, Action::Press, modifiers) => {
                     if modifiers.contains(NATIVE_MOD) {
@@ -662,7 +688,12 @@ impl window::State for AppState {
         Option<&mut dyn kiss3d::renderer::Renderer>,
         Option<&mut dyn kiss3d::post_processing::PostProcessingEffect>,
     ) {
-        (Some(&mut self.app.viewer.arc_ball), None, None, None)
+        (
+            Some(&mut self.app.viewer.arc_ball),
+            None,
+            Some(&mut self.point_cloud_renderer),
+            None,
+        )
     }
 }
 
